@@ -39,6 +39,7 @@
 
 #include "error.h"
 #include "format.h"
+#include "glcontainers.h"
 #include "paths.h"
 
 static size_t expand_hex (char **str)
@@ -131,7 +132,6 @@ static void load_all_formats (gl_list_t *formats)
 gl_list_t find_interpreters (const char *path)
 {
     gl_list_t formats, ok_formats, interpreters;
-    gl_list_iterator_t format_iter;
     const struct binfmt *binfmt;
     size_t toread;
     char *buf;
@@ -145,9 +145,7 @@ gl_list_t find_interpreters (const char *path)
      * about huge memory consumption.
      */
     toread = 0;
-    format_iter = gl_list_iterator (formats);
-    while (gl_list_iterator_next (&format_iter, (const void **) &binfmt,
-				  NULL)) {
+    GL_LIST_FOREACH_START (formats, binfmt) {
 	if (!strcmp (binfmt->type, "magic")) {
 	    size_t size;
 
@@ -155,8 +153,7 @@ gl_list_t find_interpreters (const char *path)
 	    if (size > toread)
 		toread = size;
 	}
-    }
-    gl_list_iterator_free (&format_iter);
+    } GL_LIST_FOREACH_END (formats);
 
     buf = xzalloc (toread);
     target_file = fopen (path, "r");
@@ -185,9 +182,7 @@ gl_list_t find_interpreters (const char *path)
 	extension = dot + 1;
 
     ok_formats = gl_list_create_empty (GL_ARRAY_LIST, NULL, NULL, NULL, true);
-    format_iter = gl_list_iterator (formats);
-    while (gl_list_iterator_next (&format_iter, (const void **) &binfmt,
-				  NULL)) {
+    GL_LIST_FOREACH_START (formats, binfmt) {
 	if (!strcmp (binfmt->type, "magic")) {
 	    char *segment;
 
@@ -203,8 +198,7 @@ gl_list_t find_interpreters (const char *path)
 	    if (extension && !strcmp (extension, binfmt->magic))
 		gl_list_add_last (ok_formats, binfmt);
 	}
-    }
-    gl_list_iterator_free (&format_iter);
+    } GL_LIST_FOREACH_END (formats);
 
     /* Everything in ok_formats is now a candidate.  Loop through twice,
      * once to try everything with a detector and once to try everything
@@ -212,9 +206,7 @@ gl_list_t find_interpreters (const char *path)
      */
     interpreters = gl_list_create_empty (GL_ARRAY_LIST, NULL, NULL, NULL,
 					 true);
-    format_iter = gl_list_iterator (ok_formats);
-    while (gl_list_iterator_next (&format_iter, (const void **) &binfmt,
-				  NULL)) {
+    GL_LIST_FOREACH_START (ok_formats, binfmt) {
 	if (*binfmt->detector) {
 	    pipeline *detector;
 
@@ -223,16 +215,12 @@ gl_list_t find_interpreters (const char *path)
 	    if (pipeline_run (detector) == 0)
 		gl_list_add_last (interpreters, binfmt);
 	}
-    }
-    gl_list_iterator_free (&format_iter);
+    } GL_LIST_FOREACH_END (ok_formats);
 
-    format_iter = gl_list_iterator (ok_formats);
-    while (gl_list_iterator_next (&format_iter, (const void **) &binfmt,
-				  NULL)) {
+    GL_LIST_FOREACH_START (ok_formats, binfmt) {
 	if (!*binfmt->detector)
 	    gl_list_add_last (interpreters, binfmt);
-    }
-    gl_list_iterator_free (&format_iter);
+    } GL_LIST_FOREACH_END (ok_formats);
 
     return interpreters;
 }
