@@ -29,12 +29,12 @@
 #include <string.h>
 #include <ctype.h>
 
-#include "gl_list.h"
+#include "gl_map.h"
 #include "xalloc.h"
 
 #include "error.h"
-#include "kvhash.h"
 #include "format.h"
+#include "glcontainers.h"
 
 struct binfmt *binfmt_load (const char *name, const char *filename, int quiet)
 {
@@ -85,7 +85,7 @@ struct binfmt *binfmt_load (const char *name, const char *filename, int quiet)
     return binfmt;
 }
 
-struct binfmt *binfmt_new (const char *name, gl_list_t args)
+struct binfmt *binfmt_new (const char *name, gl_map_t args)
 {
     struct binfmt *binfmt;
 
@@ -93,7 +93,7 @@ struct binfmt *binfmt_new (const char *name, gl_list_t args)
     binfmt->name = xstrdup (name);
 
 #define SET_FIELD(field) do { \
-    const char *value = kvhash_lookup (args, #field); \
+    const char *value = gl_map_get (args, #field); \
     /* value may be NULL, as update-binfmts' parser is simpler that way. */ \
     if (value && strchr (value, '\n')) \
 	quit ("newlines prohibited in binfmt files (%s)", value); \
@@ -115,13 +115,13 @@ struct binfmt *binfmt_new (const char *name, gl_list_t args)
 
     if (!binfmt->type) {
 	if (binfmt->magic) {
-	    if (kvhash_exists (args, "extension")) {
+	    if (map_contains (args, "extension")) {
 		warning ("%s: can't use both --magic and --extension", name);
 		return NULL;
 	    } else
 		binfmt->type = xstrdup ("magic");
 	} else {
-	    if (kvhash_exists (args, "extension"))
+	    if (map_contains (args, "extension"))
 		binfmt->type = xstrdup ("extension");
 	    else {
 		warning ("%s: either --magic or --extension is required",
@@ -135,7 +135,7 @@ struct binfmt *binfmt_new (const char *name, gl_list_t args)
 	const char *extension;
 
 	free (binfmt->magic);
-	extension = kvhash_lookup (args, "extension");
+	extension = gl_map_get (args, "extension");
 	if (extension && strchr (extension, '\n'))
 	    quit ("newlines prohibited in binfmt files (%s)", extension);
 	binfmt->magic = extension ? xstrdup (extension) : NULL;
@@ -238,10 +238,3 @@ void binfmt_free (struct binfmt *binfmt)
     free (binfmt->fix_binary);
     free (binfmt);
 }
-
-void kvhash_dispose_binfmt (void *data)
-{
-    struct kvelem *kvdata = data;
-    binfmt_free ((struct binfmt *) kvdata->value);
-    kvhash_dispose (data);
-}
