@@ -31,6 +31,7 @@
 #include "error.h"
 #include "find.h"
 #include "format.h"
+#include "glcontainers.h"
 #include "paths.h"
 
 char *program_name;
@@ -82,7 +83,6 @@ int main (int argc, char **argv)
     char **real_argv;
     int i;
     gl_list_t interpreters;
-    gl_list_iterator_t interpreter_iter;
     const struct binfmt *binfmt;
 
     program_name = xstrdup ("run-detectors");
@@ -101,15 +101,12 @@ int main (int argc, char **argv)
     interpreters = find_interpreters (argv[arg_index]);
 
     /* Try to exec() each interpreter in turn. */
-    interpreter_iter = gl_list_iterator (interpreters);
-    while (gl_list_iterator_next (&interpreter_iter, (const void **) &binfmt,
-				  NULL)) {
+    GL_LIST_FOREACH_START (interpreters, binfmt) {
 	real_argv[0] = (char *) binfmt->interpreter;
 	fflush (NULL);
 	execvp (binfmt->interpreter, real_argv);
 	warning_err ("unable to exec %s", binfmt->interpreter);
-    }
-    gl_list_iterator_free (&interpreter_iter);
+    } GL_LIST_FOREACH_END (interpreters);
 
     gl_list_free (interpreters);
 
