@@ -46,14 +46,19 @@
 #include <pipeline.h>
 
 #include "argp.h"
+#include "gl_hash_map.h"
 #include "gl_list.h"
+#include "gl_rbtree_omap.h"
+#include "gl_xmap.h"
+#include "gl_xomap.h"
 #include "xalloc.h"
+#include "xstdopen.h"
 #include "xvasprintf.h"
 
 #include "error.h"
 #include "find.h"
 #include "format.h"
-#include "kvhash.h"
+#include "glcontainers.h"
 #include "paths.h"
 
 char *program_name;
@@ -63,7 +68,7 @@ static int test = 0;
 static char *path_register, *path_status;
 static char *run_detectors;
 
-gl_list_t formats;
+gl_omap_t formats;
 
 static inline bool exists (const char *name)
 {
@@ -114,10 +119,10 @@ static int rename_mv (const char *source, const char *dest)
     return pipeline_run (mv) == 0;
 }
 
-static gl_list_t get_import (const char *name)
+static gl_map_t get_import (const char *name)
 {
     FILE *import_file;
-    gl_list_t import;
+    gl_map_t import;
     char *line = NULL;
     ssize_t len;
     size_t n;
@@ -127,8 +132,7 @@ static gl_list_t get_import (const char *name)
 	warning_err ("unable to open %s", name);
 	return NULL;
     }
-    import = kvhash_initialize
-	((gl_listelement_dispose_fn) kvhash_dispose_plain);
+    import = new_string_map (GL_HASH_MAP, plain_free);
 
     while ((len = getline (&line, &n, import_file)) != -1) {
 	char *space, *key, *value, *p, *q;
@@ -149,9 +153,8 @@ static gl_list_t get_import (const char *name)
 	    *p++ = tolower ((int) *q++);
 	*p = 0;
 
-	if (kvhash_insert (import, key, value) != value)
-	    free (value);
-	free (key);
+	/* Steals memory for key and value. */
+	gl_map_put (import, key, value);
 	free (line);
 	line = NULL;
     }
@@ -256,10 +259,8 @@ static void load_format (const char *name, int quiet)
     char *admindir_name = xasprintf ("%s/%s", admindir, name);
     if (is_file (admindir_name)) {
 	struct binfmt *format = binfmt_load (name, admindir_name, quiet);
-	if (format) {
-	    if (kvhash_insert (formats, name, format) != format)
-		binfmt_free (format);
-	}
+	if (format && !omap_contains (formats, name))
+	    gl_omap_put (formats, xstrdup (name), format);
     }
     free (admindir_name);
 }
@@ -286,12 +287,12 @@ static void load_all_formats (int quiet)
 static int act_enable (const char *name)
 {
     char *procdir_name;
+    const struct binfmt *binfmt;
 
     if (!load_binfmt_misc ())
 	return 1;
 
     if (name) {
-	const struct binfmt *binfmt;
 	char type;
 	int need_detector;
 	const char *interpreter;
@@ -316,17 +317,17 @@ static int act_enable (const char *name)
 	free (procdir_name);
 
 	load_format (name, 0);
-	if (!test && !kvhash_exists (formats, name)) {
+	binfmt = gl_omap_get (formats, name);
+	if (!test && !binfmt) {
 	    warning ("%s not in database of installed binary formats.", name);
 	    return 0;
 	}
-	binfmt = kvhash_lookup (formats, name);
 	type = (!strcmp (binfmt->type, "magic")) ? 'M' : 'E';
 
 	need_detector = binfmt->detector && *binfmt->detector;
 	if (!need_detector) {
-	    gl_list_iterator_t format_iter;
-	    const void *format_elem;
+	    const char *other_name;
+	    const struct binfmt *other_binfmt;
 
 	    /* Scan the format database to see if anything else uses the
 	     * same spec as us. If so, assume that we need a detector,
@@ -336,17 +337,14 @@ static int act_enable (const char *name)
 	     */
 	    load_all_formats (1);
 
-	    format_iter = gl_list_iterator (formats);
-	    while (gl_list_iterator_next (&format_iter, &format_elem, NULL)) {
-		const struct kvelem *format_kvelem = format_elem;
-		if (!strcmp (format_kvelem->key, name))
+	    GL_OMAP_FOREACH_START (formats, other_name, other_binfmt) {
+		if (!strcmp (name, other_name))
 		    continue;
-		if (binfmt_equals (binfmt, format_kvelem->value)) {
+		if (binfmt_equals (binfmt, other_binfmt)) {
 		    need_detector = 1;
 		    break;
 		}
-	    }
-	    gl_list_iterator_free (&format_iter);
+	    } GL_OMAP_FOREACH_END (formats);
 	}
 	/* Fake the interpreter if we need a userspace detector program. */
 	interpreter = need_detector ? run_detectors : binfmt->interpreter;
@@ -390,19 +388,14 @@ static int act_enable (const char *name)
 	return 1;
     } else {
 	int worked = 1;
-	gl_list_iterator_t format_iter;
-	const void *format_elem;
 
 	load_all_formats (0);
-	format_iter = gl_list_iterator (formats);
-	while (gl_list_iterator_next (&format_iter, &format_elem, NULL)) {
-	    const struct kvelem *format_kvelem = format_elem;
-	    procdir_name = xasprintf ("%s/%s", procdir, format_kvelem->key);
+	GL_OMAP_FOREACH_START (formats, name, binfmt) {
+	    procdir_name = xasprintf ("%s/%s", procdir, name);
 	    if (!exists (procdir_name))
-		worked &= act_enable (format_kvelem->key);
+		worked &= act_enable (name);
 	    free (procdir_name);
-	}
-	gl_list_iterator_free (&format_iter);
+	} GL_OMAP_FOREACH_END (formats);
 	return worked;
     }
 }
@@ -462,20 +455,15 @@ static int act_disable (const char *name)
 	return 1;
     } else {
 	int worked = 1;
-	gl_list_iterator_t format_iter;
-	const void *format_elem;
+	const struct binfmt *binfmt;
 
 	load_all_formats (0);
-	format_iter = gl_list_iterator (formats);
-	while (gl_list_iterator_next (&format_iter, &format_elem, NULL)) {
-	    const struct kvelem *format_kvelem = format_elem;
-	    char *procdir_id = xasprintf ("%s/%s", procdir,
-					  format_kvelem->key);
+	GL_OMAP_FOREACH_START (formats, name, binfmt) {
+	    char *procdir_id = xasprintf ("%s/%s", procdir, name);
 	    if (exists (procdir_id))
-		worked &= act_disable (format_kvelem->key);
+		worked &= act_disable (name);
 	    free (procdir_id);
-	}
-	gl_list_iterator_free (&format_iter);
+	} GL_OMAP_FOREACH_END (formats);
 	unload_binfmt_misc (); /* ignore errors here */
 	return worked;
     }
@@ -484,12 +472,12 @@ static int act_disable (const char *name)
 static int act_install (const char *name, const struct binfmt *binfmt,
 			int always_update_kernel)
 {
+    const struct binfmt *old_binfmt;
     char *admindir_name;
 
     load_format (name, 1);
-    if (kvhash_exists (formats, name)) {
-	const struct binfmt *old_binfmt = kvhash_lookup (formats, name);
-
+    old_binfmt = gl_omap_get (formats, name);
+    if (old_binfmt) {
 	/* For now we just silently zap any old versions with the same
 	 * package name (has to be silent or upgrades are annoying).  Maybe
 	 * we should be more careful in the future.
@@ -542,7 +530,7 @@ static int act_install (const char *name, const struct binfmt *binfmt,
 	free (admindir_name_tmp);
     }
     free (admindir_name);
-    kvhash_insert (formats, name, binfmt);
+    gl_omap_put (formats, xstrdup (name), binfmt);
     if (always_update_kernel || is_enabled ()) {
 	if (!act_enable (name)) {
 	    warning ("unable to enable binary format %s", name);
@@ -559,6 +547,7 @@ static int act_remove (const char *name, const char *package,
 		       int always_update_kernel)
 {
     char *admindir_name;
+    const struct binfmt *old_binfmt;
 
     admindir_name = xasprintf ("%s/%s", admindir, name);
     if (!is_file (admindir_name)) {
@@ -571,8 +560,8 @@ static int act_remove (const char *name, const char *package,
 	return 1;
     }
     load_format (name, 1);
-    if (kvhash_exists (formats, name)) {
-	const struct binfmt *old_binfmt = kvhash_lookup (formats, name);
+    old_binfmt = gl_omap_get (formats, name);
+    if (old_binfmt) {
 	const char *old_package = old_binfmt->package;
 	if (strcmp (package, old_package)) {
 	    if (!strcmp (package, ":"))
@@ -601,7 +590,7 @@ static int act_remove (const char *name, const char *package,
 	    free (admindir_name);
 	    return 0;
 	}
-	kvhash_delete (formats, name);
+	gl_omap_remove (formats, name);
     }
     free (admindir_name);
     return 1;
@@ -615,7 +604,8 @@ static int act_import (const char *name)
     if (name) {
 	const char *slash, *id;
 	char *path;
-	gl_list_t import;
+	gl_map_t import;
+	const struct binfmt *binfmt;
 	const char *interpreter;
 
 	slash = strrchr (name, '/');
@@ -635,15 +625,15 @@ static int act_import (const char *name)
 	}
 
 	import = get_import (path);
-	if (!import || !gl_list_size (import)) {
+	if (!import || !gl_map_size (import)) {
 	    warning ("couldn't find information about '%s' to import", id);
 	    free (path);
 	    return 0;
 	}
 
 	load_format (id, 1);
-	if (kvhash_exists (formats, id)) {
-	    const struct binfmt *binfmt = kvhash_lookup (formats, id);
+	binfmt = gl_omap_get (formats, id);
+	if (binfmt) {
 	    if (!strcmp (binfmt->package, ":")) {
 		/* Installed version was installed manually, so don't import
 		 * over it.
@@ -659,18 +649,19 @@ static int act_import (const char *name)
 	}
 
 	/* TODO: This duplicates the verification code below slightly. */
-	if (!kvhash_exists (import, "package")) {
+	if (!map_contains (import, "package")) {
 	    warning ("%s: required 'package' line missing", path);
 	    free (path);
 	    return 0;
 	}
 
-	interpreter = kvhash_lookup (import, "interpreter");
+	interpreter = gl_map_get (import, "interpreter");
 	if (!interpreter || access (interpreter, X_OK))
 	    warning ("%s: no executable %s found, but continuing anyway as "
 		     "you request", path, interpreter);
 
 	act_install (id, binfmt_new (path, import), 0);
+	gl_map_free (import);
 	free (path);
 	return 1;
     } else {
@@ -707,7 +698,8 @@ static int act_unimport (const char *name)
     if (name) {
 	const char *slash, *id;
 	char *path;
-	gl_list_t import;
+	gl_map_t import;
+	const struct binfmt *binfmt;
 	const char *package;
 
 	slash = strrchr (name, '/');
@@ -727,15 +719,15 @@ static int act_unimport (const char *name)
 	}
 
 	import = get_import (path);
-	if (!import || !gl_list_size (import)) {
+	if (!import || !gl_map_size (import)) {
 	    warning ("couldn't find information about '%s' to unimport", id);
 	    free (path);
 	    return 0;
 	}
 
 	load_format (id, 1);
-	if (kvhash_exists (formats, id)) {
-	    const struct binfmt *binfmt = kvhash_lookup (formats, id);
+	binfmt = gl_omap_get (formats, id);
+	if (binfmt) {
 	    if (!strcmp (binfmt->package, ":")) {
 		/* Installed version was installed manually, so don't
 		 * unimport it.
@@ -750,7 +742,7 @@ static int act_unimport (const char *name)
 	    return 1;
 	}
 
-	package = kvhash_lookup (import, "package");
+	package = gl_map_get (import, "package");
 	if (!package) {
 	    warning ("%s: required 'package' line missing", path);
 	    free (path);
@@ -758,6 +750,7 @@ static int act_unimport (const char *name)
 	}
 
 	act_remove (id, package, 0);
+	gl_map_free (import);
 	free (path);
 	return 1;
     } else {
@@ -787,14 +780,15 @@ static int act_unimport (const char *name)
 
 static int act_display (const char *name)
 {
+    const struct binfmt *binfmt;
+
     if (name) {
 	char *procdir_name;
-	const struct binfmt *binfmt;
 	const char *package;
 
 	procdir_name = xasprintf ("%s/%s", procdir, name);
 	load_format (name, 0);
-	binfmt = kvhash_lookup (formats, name);
+	binfmt = gl_omap_get (formats, name);
 	if (!binfmt) {
 	    warning ("%s not in database of installed binary formats.", name);
 	    return 0;
@@ -814,14 +808,10 @@ static int act_display (const char *name)
 	    package, binfmt->type, binfmt->offset, binfmt->magic, binfmt->mask,
 	    binfmt->interpreter, binfmt->detector);
     } else {
-	gl_list_iterator_t format_iter;
-	const void *format_elem;
-
 	load_all_formats (0);
-	format_iter = gl_list_iterator (formats);
-	while (gl_list_iterator_next (&format_iter, &format_elem, NULL))
-	    act_display (((const struct kvelem *) format_elem)->key);
-	gl_list_iterator_free (&format_iter);
+	GL_OMAP_FOREACH_START (formats, name, binfmt)
+	    act_display (name);
+	GL_OMAP_FOREACH_END (formats);
     }
     return 1;
 }
@@ -829,16 +819,13 @@ static int act_display (const char *name)
 static int act_find (const char *executable)
 {
     gl_list_t interpreters;
-    gl_list_iterator_t interpreter_iter;
     const struct binfmt *binfmt;
 
     interpreters = find_interpreters (executable);
 
-    interpreter_iter = gl_list_iterator (interpreters);
-    while (gl_list_iterator_next (&interpreter_iter, (const void **) &binfmt,
-				  NULL))
+    GL_LIST_FOREACH_START (interpreters, binfmt)
 	printf ("%s\n", binfmt->interpreter);
-    gl_list_iterator_free (&interpreter_iter);
+    GL_LIST_FOREACH_END (interpreters);
 
     gl_list_free (interpreters);
 
@@ -1153,6 +1140,8 @@ int main (int argc, char **argv)
 
     program_name = xstrdup ("update-binfmts");
 
+    xstdopen ();
+
     check_supported_os ();
 
     argp_err_exit_status = 2;
@@ -1166,21 +1155,21 @@ int main (int argc, char **argv)
     if (!package)
 	package = ":";
 
-    formats = kvhash_initialize
-	((gl_listelement_dispose_fn) kvhash_dispose_binfmt);
+    formats = new_string_omap (GL_RBTREE_OMAP,
+			       (gl_mapvalue_dispose_fn) binfmt_free);
 
     if (mode == OPT_INSTALL) {
 	struct binfmt *binfmt;
-	gl_list_t format_args;
+	gl_map_t format_args;
 
-	format_args = kvhash_initialize
-	    ((gl_listelement_dispose_fn) kvhash_dispose_plain);
-	kvhash_insert (format_args, "package", package);
-	kvhash_insert (format_args, "type",
-		       (type == OPT_MAGIC) ? "magic" : "extension");
+	format_args = gl_map_create_empty (GL_HASH_MAP, string_equals,
+					   string_hash, NULL, NULL);
+	gl_map_put (format_args, "package", package);
+	gl_map_put (format_args, "type",
+		    (type == OPT_MAGIC) ? "magic" : "extension");
 #define ADD_SPEC(field) do { \
     if (spec.field) \
-	kvhash_insert (format_args, #field, spec.field); \
+	gl_map_put (format_args, #field, spec.field); \
 } while (0)
 	ADD_SPEC (magic);
 	ADD_SPEC (mask);
